@@ -1,5 +1,11 @@
-{-# LANGUAGE BangPatterns, CPP, DeriveDataTypeable, MagicHash, Rank2Types,
-    ScopedTypeVariables #-}
+{-# LANGUAGE BangPatterns        #-}
+{-# LANGUAGE CPP                 #-}
+{-# LANGUAGE DeriveDataTypeable  #-}
+{-# LANGUAGE MagicHash           #-}
+{-# LANGUAGE Rank2Types          #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeFamilies        #-}
+{-# LANGUAGE FlexibleContexts    #-}
 -- |
 -- Module    : System.Random.MWC
 -- Copyright : (c) 2009, 2010 Bryan O'Sullivan
@@ -21,6 +27,8 @@ module System.Random.MWC
     (
     -- * Types
       Gen
+    , GenIO
+    , GenST
     , Seed
     , Variate(..)
     -- * Other distributions
@@ -42,27 +50,35 @@ module System.Random.MWC
 #include "MachDeps.h"
 #endif
 
-import Control.Exception (IOException, catch)
-import Control.Monad (ap, liftM, unless)
+import Control.Exception       (IOException, catch)
+import Control.Monad           (ap, liftM, unless)
 import Control.Monad.Primitive (PrimMonad, PrimState, unsafePrimToIO)
-import Data.Bits ((.&.), (.|.), xor)
-import Data.IORef (atomicModifyIORef, newIORef)
-import Data.Int (Int8, Int16, Int32, Int64)
-import Data.Ratio ((%), numerator)
-import Data.Time.Clock.POSIX (getPOSIXTime)
-import Data.Typeable (Typeable)
-import Data.Vector.Generic (Vector, unsafeFreeze)
-import Data.Word (Word, Word8, Word16, Word32, Word64)
-import Foreign.Marshal.Alloc (allocaBytes)
-import Foreign.Marshal.Array (peekArray)
-import GHC.Base (Int(I#))
-import GHC.Word (Word64(W64#), uncheckedShiftL64#, uncheckedShiftRL64#)
+import Control.Monad.ST        (ST)
+import Data.Bits               ((.&.), (.|.), xor)
+import Data.Int                (Int8, Int16, Int32, Int64)
+import Data.IORef              (atomicModifyIORef, newIORef)
+import Data.Ratio              ((%), numerator)
+import Data.Time.Clock.POSIX   (getPOSIXTime)
+import Data.Typeable           (Typeable)
+import Data.Vector.Generic     (Vector, unsafeFreeze)
+import Data.Word               (Word, Word8, Word16, Word32, Word64)
+import Foreign.Marshal.Alloc   (allocaBytes)
+import Foreign.Marshal.Array   (peekArray)
+import GHC.Base       (Int(I#))
+import GHC.Word       (Word64(W64#), uncheckedShiftL64#, uncheckedShiftRL64#)
 import Prelude hiding (catch)
-import System.CPUTime (cpuTimePrecision, getCPUTime)
-import System.IO (IOMode(..), hGetBuf, hPutStrLn, stderr, withBinaryFile)
-import System.IO.Unsafe (unsafePerformIO)
-import qualified Data.Vector.Unboxed as I
+import qualified Data.Vector.Generic         as G
+import qualified Data.Vector.Generic.Mutable as GM
+import qualified Data.Vector.Unboxed         as I
 import qualified Data.Vector.Unboxed.Mutable as M
+import System.CPUTime   (cpuTimePrecision, getCPUTime)
+import System.IO        (IOMode(..), hGetBuf, hPutStrLn, stderr, withBinaryFile)
+import System.IO.Unsafe (unsafePerformIO)
+
+-- FIXME: removal of Unbox constraint leads to severe (~10x)
+--        performance drop with GHC 6.12. For details see bug #33 in the 
+--        vector bug tracker[1]
+-- [1] http://trac.haskell.org/vector/ticket/33
 
 -- | The class of types for which we can generate uniformly
 -- distributed random variates.
@@ -93,97 +109,137 @@ class M.Unbox a => Variate a where
     -- 2**(-33).  To do the same with 'Double' variates, subtract
     -- 2**(-53).
     uniform :: (PrimMonad m) => Gen (PrimState m) -> m a
-
--- Thanks to Duncan Coutts for finding the pattern below for
--- strong-arming GHC 6.10's inliner into behaving itself.  This makes
--- a 2x difference to performance compared to the following:
---
--- > uniform = uniform1 fromIntegral
+    -- | Generate single uniformly distributed random variable in a
+    -- given range.
+    --
+    -- * For integral types inclusive range is used.
+    --
+    -- * For floating point numbers range (a,b] is used if one ignores
+    --   rounding errors.
+    uniformR :: (PrimMonad m) => (a,a) -> Gen (PrimState m) -> m a
 
 instance Variate Int8 where
-    uniform = f where f = uniform1 fromIntegral
-                      {-# INLINE f #-}
+    uniform  = uniform1 fromIntegral
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Int16 where
-    uniform = f where f = uniform1 fromIntegral
-                      {-# INLINE f #-}
+    uniform  = uniform1 fromIntegral
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Int32 where
-    uniform = f where f = uniform1 fromIntegral
-                      {-# INLINE f #-}
+    uniform  = uniform1 fromIntegral
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Int64 where
-    uniform = f where f = uniform2 wordsTo64Bit
-                      {-# INLINE f #-}
+    uniform  = uniform2 wordsTo64Bit
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Word8 where
-    uniform = f where f = uniform1 fromIntegral
-                      {-# INLINE f #-}
+    uniform  = uniform1 fromIntegral
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Word16 where
-    uniform = f where f = uniform1 fromIntegral
-                      {-# INLINE f #-}
+    uniform  = uniform1 fromIntegral
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Word32 where
-    uniform = uniformWord32
+    uniform  = uniform1 fromIntegral
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Word64 where
-    uniform = f where f = uniform2 wordsTo64Bit
-                      {-# INLINE f #-}
+    uniform  = uniform2 wordsTo64Bit
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Bool where
-    uniform = f where f = uniform1 wordToBool
-                      {-# INLINE f #-}
+    uniform = uniform1 wordToBool
+    uniformR (False,True)  g = uniform g
+    uniformR (False,False) _ = return False
+    uniformR (True,True)   _ = return True
+    uniformR (True,False)  g = uniform g
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Float where
-    uniform = f where f = uniform1 wordToFloat
-                      {-# INLINE f #-}
+    uniform          = uniform1 wordToFloat
+    uniformR (x1,x2) = uniform1 (\w -> x1 + (x2-x1) * wordToFloat w)
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Double where
-    uniform = f where f = uniform2 wordsToDouble
-                      {-# INLINE f #-}
+    uniform          = uniform2 wordsToDouble
+    uniformR (x1,x2) = uniform2 (\w1 w2 -> x1 + (x2-x1) * wordsToDouble w1 w2)
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Int where
 #if WORD_SIZE_IN_BITS < 64
-    uniform = f where f = uniform1 fromIntegral
+    uniform = uniform1 fromIntegral
 #else
-    uniform = f where f = uniform2 wordsTo64Bit
+    uniform = uniform2 wordsTo64Bit
 #endif
-                      {-# INLINE f #-}
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance Variate Word where
 #if WORD_SIZE_IN_BITS < 64
-    uniform = f where f = uniform1 fromIntegral
+    uniform = uniform1 fromIntegral
 #else
-    uniform = f where f = uniform2 wordsTo64Bit
+    uniform = uniform2 wordsTo64Bit
 #endif
-                      {-# INLINE f #-}
+    uniformR = uniformRange
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 {-
 instance Variate Integer where
-    uniform = f where f g = do
-                           u <- uniform g
-                           return $! fromIntegral (u :: Int)
-                      {-# INLINE f #-}
+    uniform g = do
+      u <- uniform g
+      return $! fromIntegral (u :: Int)
+    {-# INLINE uniform #-}
 -}
 
 instance (Variate a, Variate b) => Variate (a,b) where
-    uniform = f where f g = (,) `liftM` uniform g `ap` uniform g
-                      {-# INLINE f #-}
+    uniform g = (,) `liftM` uniform g `ap` uniform g
+    uniformR ((x1,y1),(x2,y2)) g = (,) `liftM` uniformR (x1,x2) g `ap` uniformR (y1,y2) g
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance (Variate a, Variate b, Variate c) => Variate (a,b,c) where
-    uniform = f where f g = (,,) `liftM` uniform g `ap` uniform g `ap` uniform g
-                      {-# INLINE f #-}
+    uniform g = (,,) `liftM` uniform g `ap` uniform g `ap` uniform g
+    uniformR ((x1,y1,z1),(x2,y2,z2)) g =
+      (,,) `liftM` uniformR (x1,x2) g `ap` uniformR (y1,y2) g `ap` uniformR (z1,z2) g
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 instance (Variate a, Variate b, Variate c, Variate d) => Variate (a,b,c,d) where
-    uniform = f
-        where f g = (,,,) `liftM` uniform g `ap` uniform g `ap` uniform g
-                          `ap` uniform g
-              {-# INLINE f #-}
+    uniform g = (,,,) `liftM` uniform g `ap` uniform g `ap` uniform g
+                `ap` uniform g
+    uniformR ((x1,y1,z1,t1),(x2,y2,z2,t2)) g =
+      (,,,) `liftM` uniformR (x1,x2) g `ap` uniformR (y1,y2) g `ap`
+                    uniformR (z1,z2) g `ap` uniformR (t1,t2) g
+    {-# INLINE uniform  #-}
+    {-# INLINE uniformR #-}
 
 wordsTo64Bit :: Integral a => Word32 -> Word32 -> a
-wordsTo64Bit a b =
-    fromIntegral ((fromIntegral a `shiftL` 32) .|. fromIntegral b)
+wordsTo64Bit x y =
+    fromIntegral ((fromIntegral x `shiftL` 32) .|. fromIntegral y)
 {-# INLINE wordsTo64Bit #-}
 
 wordToBool :: Word32 -> Bool
@@ -198,18 +254,24 @@ wordToFloat x      = (fromIntegral i * m_inv_32) + 0.5 + m_inv_33
 {-# INLINE wordToFloat #-}
 
 wordsToDouble :: Word32 -> Word32 -> Double
-wordsToDouble x y  = (fromIntegral a * m_inv_32 + (0.5 + m_inv_53) +
-                     fromIntegral (b .&. 0xFFFFF) * m_inv_52) 
+wordsToDouble x y  = (fromIntegral u * m_inv_32 + (0.5 + m_inv_53) +
+                     fromIntegral (v .&. 0xFFFFF) * m_inv_52) 
     where m_inv_52 = 2.220446049250313080847263336181640625e-16
           m_inv_53 = 1.1102230246251565404236316680908203125e-16
           m_inv_32 = 2.3283064365386962890625e-10
-          a        = fromIntegral x :: Int32
-          b        = fromIntegral y :: Int32
+          u        = fromIntegral x :: Int32
+          v        = fromIntegral y :: Int32
 {-# INLINE wordsToDouble #-}
 
 -- | State of the pseudo-random number generator.
 newtype Gen s = Gen (M.MVector s Word32)
 
+-- | A shorter name for PRNG state in the IO monad.
+type GenIO = Gen (PrimState IO)
+
+-- | A shorter name for PRNG state in the ST monad.
+type GenST s = Gen (PrimState (ST s))
+
 ioff, coff :: Int
 ioff = 256
 coff = 257
@@ -255,9 +317,19 @@ initialize seed = do
 newtype Seed = Seed (I.Vector Word32)
     deriving (Eq, Show, Typeable)
 
+-- Safe version of unsafeFreeze.
+-- NOTE: vector-0.7 will provide function `freeze' with same
+--       functionality. This function shall be removed when support for
+--       vector<=0.6 is dropped
+safeFreeze :: (PrimMonad m, Vector v a) => G.Mutable v (PrimState m) a -> m (v a)
+safeFreeze v = do
+  v' <- GM.unsafeNew (GM.length v)
+  GM.unsafeCopy v' v
+  unsafeFreeze v'
+
 -- | Save the state of a 'Gen', for later use by 'restore'.
 save :: PrimMonad m => Gen (PrimState m) -> m Seed
-save (Gen q) = Seed `liftM` unsafeFreeze q
+save (Gen q) = Seed `liftM` safeFreeze q
 {-# INLINE save #-}
 
 -- | Create a new 'Gen' that mirrors the state of a saved 'Seed'.
@@ -321,18 +393,26 @@ nextIndex :: Integral a => a -> Int
 nextIndex i = fromIntegral j
     where j = fromIntegral (i+1) :: Word8
 
+a :: Word64
+a = 1540315826
+
 uniformWord32 :: PrimMonad m => Gen (PrimState m) -> m Word32
 uniformWord32 (Gen q) = do
-  let a = 809430660 :: Word64
-  i <- nextIndex `liftM` M.unsafeRead q ioff
-  c <- fromIntegral `liftM` M.unsafeRead q coff
+  i  <- nextIndex `liftM` M.unsafeRead q ioff
+  c  <- fromIntegral `liftM` M.unsafeRead q coff
   qi <- fromIntegral `liftM` M.unsafeRead q i
-  let t   = a * qi + c
-      t32 = fromIntegral t
-  M.unsafeWrite q i t32
+  let t  = a * qi + c
+      c' = fromIntegral (t `shiftR` 32)
+      x  = fromIntegral t + c'
+      x_lt_c = x < c'
+      x'  | x_lt_c    = x + 1
+          | otherwise = x
+      c'' | x_lt_c    = c' + 1
+          | otherwise = c'
+  M.unsafeWrite q i x'
   M.unsafeWrite q ioff (fromIntegral i)
-  M.unsafeWrite q coff (fromIntegral (t `shiftR` 32))
-  return t32
+  M.unsafeWrite q coff (fromIntegral c'')
+  return x'
 {-# INLINE uniformWord32 #-}
 
 uniform1 :: PrimMonad m => (Word32 -> a) -> Gen (PrimState m) -> m a
@@ -343,33 +423,92 @@ uniform1 f gen = do
 
 uniform2 :: PrimMonad m => (Word32 -> Word32 -> a) -> Gen (PrimState m) -> m a
 uniform2 f (Gen q) = do
-  let a = 809430660 :: Word64
-  i <- nextIndex `liftM` M.unsafeRead q ioff
+  i  <- nextIndex `liftM` M.unsafeRead q ioff
   let j = nextIndex i
-  c <- fromIntegral `liftM` M.unsafeRead q coff
+  c  <- fromIntegral `liftM` M.unsafeRead q coff
   qi <- fromIntegral `liftM` M.unsafeRead q i
   qj <- fromIntegral `liftM` M.unsafeRead q j
   let t   = a * qi + c
-      t32 = fromIntegral t
-      c'  = t `shiftR` 32
-      u   = a * qj + c'
-      u32 = fromIntegral u
-  M.unsafeWrite q i t32
-  M.unsafeWrite q j u32
+      c'  = fromIntegral (t `shiftR` 32)
+      x   = fromIntegral t + c'
+      x_lt_c = x < c'
+      x'  | x_lt_c    = x + 1
+          | otherwise = x
+      c'' | x_lt_c    = c' + 1
+          | otherwise = c'
+      u   = a * qj + fromIntegral c''
+      d'  = fromIntegral (u `shiftR` 32)
+      y   = fromIntegral u + d'
+      y_lt_d = y < d'
+      y'  | y_lt_d    = y + 1
+          | otherwise = y
+      d'' | y_lt_d    = d' + 1
+          | otherwise = d'
+  M.unsafeWrite q i x'
+  M.unsafeWrite q j y'
   M.unsafeWrite q ioff (fromIntegral j)
-  M.unsafeWrite q coff (fromIntegral (u `shiftR` 32))
-  return $! f t32 u32
+  M.unsafeWrite q coff (fromIntegral d'')
+  return $! f x' y'
 {-# INLINE uniform2 #-}
 
+-- Type family for fixed size integrals. For signed data types it's
+-- its unsigned couterpart with same size and for unsigned data types
+-- it's same type
+type family Unsigned a :: *
+
+type instance Unsigned Int8  = Word8
+type instance Unsigned Int16 = Word16
+type instance Unsigned Int32 = Word32
+type instance Unsigned Int64 = Word64
+type instance Unsigned Int   = Word
+
+type instance Unsigned Word8  = Word8
+type instance Unsigned Word16 = Word16
+type instance Unsigned Word32 = Word32
+type instance Unsigned Word64 = Word64
+type instance Unsigned Word   = Word
+
+-- Subtract two numbers under assumption that x>=y and store result in
+-- unsigned data type of same size
+sub :: (Integral a, Integral (Unsigned a)) => a -> a -> Unsigned a
+sub x y = fromIntegral x - fromIntegral y
+
+add :: (Integral a, Integral (Unsigned a)) => a -> Unsigned a -> a
+add m x = m + fromIntegral x
+
+-- Generate uniform value in the range [0,n). Values must be
+-- unsigned. Second parameter is random number generator
+unsignedRange :: (PrimMonad m, Integral a, Bounded a) => a -> m a -> m a
+unsignedRange n rnd = go
+  where
+    buckets = maxBound `div` n
+    maxN    = buckets * n
+    go = do x <- rnd
+            if x < maxN then return (x `div` buckets)
+                        else go
+{-# INLINE unsignedRange #-}
+
+-- Generate unformly distributed value in inclusive range.
+uniformRange :: ( PrimMonad m
+                , Integral a, Bounded a, Variate a
+                , Integral (Unsigned a), Bounded (Unsigned a), Variate (Unsigned a))
+             => (a,a) -> Gen (PrimState m) -> m a
+uniformRange (x1,x2) g
+  | x1 == minBound && x2 == maxBound = uniform g
+  | otherwise                        = do x <- unsignedRange (sub x2 x1 + 1) (uniform g)
+                                          return $! add x1 x
+{-# INLINE uniformRange #-}
+
 -- | Generate a vector of pseudo-random variates.  This is not
 -- necessarily faster than invoking 'uniform' repeatedly in a loop,
 -- but it may be more convenient to use in some situations.
 uniformVector :: (PrimMonad m, Variate a)
              => Gen (PrimState m) -> Int -> m (I.Vector a)
-uniformVector gen n = M.unsafeNew n >>= go (uniform gen) 0
-  where
-    go u !i mu | i >= n    = unsafeFreeze mu
-               | otherwise = u >>= M.unsafeWrite mu i >> go u (i+1) mu
+uniformVector gen n = do
+  mu <- M.unsafeNew n
+  let go !i | i < n     = uniform gen >>= M.unsafeWrite mu i >> go (i+1)
+            | otherwise = unsafeFreeze mu
+  go 0
 {-# INLINE uniformVector #-}
 
 data T = T {-# UNPACK #-} !Double {-# UNPACK #-} !Double
@@ -403,8 +542,8 @@ normal gen = loop
             then return x
             else loop
     blocks = let f = exp (-0.5 * r * r)
-             in (`I.snoc` 0) . I.cons (v/f) . I.cons r . I.take 126 .
-                I.unfoldr go $! T r f
+             in (`I.snoc` 0) . I.cons (v/f) . I.cons r .
+                I.unfoldrN 126 go $! T r f
       where
         go (T b g)   = let !u = T h (exp (-0.5 * h * h))
                            h  = sqrt (-2 * log (v / b + g))
